"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const chalk = require("chalk");
const cli_utils_1 = require("@ionic/cli-utils");
const plugins_1 = require("../lib/plugins");
let HelpCommand = class HelpCommand extends cli_utils_1.Command {
    run(inputs, options) {
        return __awaiter(this, void 0, void 0, function* () {
            // If there are no inputs then show help command help.
            if (inputs.length === 0) {
                return this.env.log.msg(cli_utils_1.formatCommandHelp(this.metadata, ''));
            }
            const [argv, command] = this.env.namespace.locateCommand(inputs);
            // If the command is located on the global namespace then show its help
            if (command) {
                return this.env.log.msg(cli_utils_1.formatCommandHelp(command.metadata, command.metadata.name));
            }
            // Resolve the plugin based on the inputs to help
            try {
                const [plugin, argv] = yield plugins_1.resolvePlugin(this.env.project.directory, inputs);
                const commandMetadataList = yield plugin.getAllCommandMetadata();
                const foundCommandList = commandMetadataList
                    .filter((cmd) => cmd.name === argv[0] || argv.length === 0);
                // No command was found if the length is zero.
                if (foundCommandList.length === 0) {
                    return this.env.log.error(`Unable to provide help on unknown command: ${chalk.bold(argv.join(' '))}`);
                }
                return foundCommandList.forEach((cmd) => {
                    const commandName = (plugin.hasOwnProperty('PLUGIN_NAME')) ? `${plugin.PLUGIN_NAME}:${cmd.name}` : cmd.name;
                    return this.env.log.msg(cli_utils_1.formatCommandHelp(cmd, commandName));
                });
            }
            catch (e) {
                return this.env.log.error(`Unable to find command: ${chalk.bold(argv.join(' '))}. It is possible that you are trying\n` +
                    `to get help on a project based command and you are not in a project directory.`);
            }
        });
    }
};
HelpCommand = __decorate([
    cli_utils_1.CommandMetadata({
        name: 'help',
        description: 'Provides help for a certain command',
        exampleCommands: ['start'],
        inputs: [
            {
                name: 'command',
                description: 'The command you desire help with',
                validators: [cli_utils_1.validators.required]
            }
        ]
    })
], HelpCommand);
exports.HelpCommand = HelpCommand;
;
