"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const fs = require("fs");
const path = require("path");
const zlib = require("zlib");
const tar = require("tar");
const chalk = require("chalk");
const cli_utils_1 = require("@ionic/cli-utils");
/**
 * Spawn an npm install task from within
 */
function pkgInstallProject(installer, root) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield cli_utils_1.runcmd(installer, ['install'], { cwd: root, stdio: 'ignore' });
        }
        catch (e) {
            throw `${installer} install failed`;
        }
    });
}
exports.pkgInstallProject = pkgInstallProject;
/**
 *
 */
function tarXvf(readStream, destination) {
    return new Promise((resolve, reject) => {
        const baseArchiveExtract = tar.Extract({
            path: destination,
            strip: 1
        })
            .on('error', reject)
            .on('end', resolve);
        try {
            readStream
                .pipe(zlib.createUnzip())
                .on('error', reject)
                .pipe(baseArchiveExtract);
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.tarXvf = tarXvf;
/**
 *
 */
function isProjectNameValid(name) {
    return name !== '.';
}
exports.isProjectNameValid = isProjectNameValid;
/**
 * If project only contains files generated by GH, it’s safe.
 * We also special case IJ-based products .idea because it integrates with CRA:
 * https://github.com/facebookincubator/create-react-app/pull/368#issuecomment-243446094
 */
function isSafeToCreateProjectIn(root) {
    var validFiles = [
        '.DS_Store', 'Thumbs.db', '.git', '.gitignore', '.idea', 'README.md', 'LICENSE'
    ];
    return fs.readdirSync(root)
        .every(function (file) {
        return validFiles.indexOf(file) >= 0;
    });
}
exports.isSafeToCreateProjectIn = isSafeToCreateProjectIn;
/**
 *
 */
function getStarterTemplateText(templateList) {
    let headerLine = chalk.bold(`Ionic Starter templates`);
    let formattedTemplateList = getStarterTemplateTextList(templateList);
    return `
    ${headerLine}
      ${formattedTemplateList.join(`
      `)}
  `;
}
exports.getStarterTemplateText = getStarterTemplateText;
function getStarterTemplateTextList(templateList) {
    return templateList.map(({ name, description }) => {
        let templateName = chalk.green(name);
        return `${templateName} ${Array(20 - name.length).join('.')} ${description}`;
    });
}
exports.getStarterTemplateTextList = getStarterTemplateTextList;
/**
 *
 */
function getHelloText() {
    return `
${chalk.bold('♬ ♫ ♬ ♫  Your Ionic app is ready to go! ♬ ♫ ♬ ♫')}

${chalk.bold('Some helpful tips:')}

${chalk.bold('Run your app in the browser (great for initial development):')}
  ionic serve

${chalk.bold('Run on a device or simulator:')}
  ionic cordova:run ios[android,browser]

${chalk.bold('Test and share your app on a device with the Ionic View app:')}
  http://view.ionic.io
  `;
}
exports.getHelloText = getHelloText;
function updateDependenciesForCLI(pathToProject, releaseChannelName = 'latest') {
    return __awaiter(this, void 0, void 0, function* () {
        const filePath = path.resolve(pathToProject, 'package.json');
        try {
            let [jsonStructure, distTags] = yield Promise.all([
                cli_utils_1.fsReadJsonFile(filePath),
                cli_utils_1.getCommandInfo('npm', ['view', '@ionic/cli-plugin-core', 'dist-tags', '--json'])
            ]);
            jsonStructure['devDependencies']['@ionic/cli-plugin-core'] = JSON.parse(distTags)[releaseChannelName];
            // TODO: eventually update ionic2-app-base and remove this.
            jsonStructure['devDependencies']['@ionic/app-scripts'] = '1.0.1';
            yield cli_utils_1.fsWriteJsonFile(filePath, jsonStructure, { encoding: 'utf8' });
        }
        catch (e) {
            if (e === cli_utils_1.ERROR_FILE_NOT_FOUND) {
                throw new Error(`${filePath} not found`);
            }
            else if (e === cli_utils_1.ERROR_FILE_INVALID_JSON) {
                throw new Error(`${filePath} is not valid JSON.`);
            }
            throw e;
        }
    });
}
exports.updateDependenciesForCLI = updateDependenciesForCLI;
